//------------------------------------------------------------
/**
 * Objet 3D au format ASE (3DSmax)
 *
 *
 * Copyright (C) 2004  Olivier Grunder
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; see the file COPYING.  If not, write to
 *   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *   Boston, MA 02111-1307, USA.
 *
 *   Contact: Olivier Grunder, 
 *            mailto:olivier.grunder@utbm.fr
 *            http://set.utbm.fr/membres/grunder/
 *
 *
 * @author      Olivier Grunder
 * @date        janvier 2004
 * @version     0.22
 */

import java.io.* ;
import gl4java.* ;

public class Objet3DASE {
  private static final String[] nomFichier = {"data/obstacle.ase",
                                              "data/nourriture.ase",
                                              "data/fourmiliere.ase",
                                              "data/fourmi.ase"} ;
  private static int MAX_ASE ;

  private static CLoadASE[] cLoadASE=null ;
  private static CLoadASE.t3DModel[] t3DModel=null ;
  private static int[] index=null ;

  //------------------------------------------------------------
  /** 
   * Constructeur par dfaut
   */
  public Objet3DASE(){
  }

  //------------------------------------------------------------
  /** 
   * Accs  un index de liste d'affichage
   */
  public static int getIndex(int i) { 
    if (i<MAX_ASE) return index[i] ; 
    return -1 ;
  }

  //------------------------------------------------------------
  /** 

   * Chargement de tous les fichiers ASE et cration des listes d'affichage
   */
   public static void init(GLFunc gl){
     if (cLoadASE==null) {
       MAX_ASE = nomFichier.length ;
       System.out.println ("Chargement des fichiers ASE 3D (MAX_ASE="+MAX_ASE+")") ;

       cLoadASE = new CLoadASE[MAX_ASE] ;
       t3DModel = new CLoadASE.t3DModel[MAX_ASE];
       index = new int[MAX_ASE] ;
       for (int i=0;i<MAX_ASE;i++) {
         // Chargement des fichiers ASE
         System.out.println ("Chargement du fichier ASE "+i+" : "+nomFichier[i]) ;
         loadFileASE (i, nomFichier[i]) ;
         
         // Liste d'affichage
         switch (i+1) {
         case Objet.OBSTACLE:
         case Objet.NOURRITURE:
           // Cration d'une liste d'affichage avec couleurs
           // provenant du fichier
           genListe(gl, i, true) ; 
           break ;
         case Objet.FOURMILIERE:
         case Objet.FOURMI:
           // Cration d'une liste d'affichage sans les couleurs
           // du fichier
           genListe(gl, i, false) ;
           break ;
         }
       }
     }
   }
     
  //------------------------------------------------------------
  /** 
   * Chargement d'un fichier ASE
   */
  public static void loadFileASE(int i, String nomFic){
    if (cLoadASE[i]==null){
      cLoadASE[i]=new CLoadASE();
      t3DModel[i] = cLoadASE[i].getModel();
      try{
        cLoadASE[i].ImportASE(t3DModel[i],nomFic);
      }
      catch (IOException ioe){
        System.err.println("Erreur d'E/S pour : "+nomFic);
      }
    }
  }

  //------------------------------------------------------------
  /** 
   * Optimisation par acceleration matrielle : 
   * cration d'une liste d'affichage
   *
   * @param gl        contexte openGL
   * @param typ       indice correspondant au type de l'objet
   * @param withColor true pour charger les couleurs du fichier.
   */
  public static void genListe(GLFunc gl, int typ, boolean withColor){
    //    System.out.println ("Generation de la liste d'affichage "+typ) ;
    CLoadASE cloadASE=cLoadASE[typ] ;
    CLoadASE.t3DModel t3Dmodel=t3DModel[typ] ;

    index[typ]=gl.glGenLists(1); //obtention d'un index mmoire
    gl.glNewList(index[typ],gl.GL_COMPILE); //debut de la liste
    //    gl.glColor3f(red, green, blue);
    for(int i = 0; i < t3Dmodel.numOfObjects; i++){
      if(t3Dmodel.pObject.size() <= 0) break;
      CLoadASE.t3DObject pObject = (CLoadASE.t3DObject)t3Dmodel.pObject.get(i);
      gl.glBegin(gl.GL_TRIANGLES);
      for(int j = 0; j < pObject.numOfFaces; j++){
        for(int whichVertex = 0; whichVertex < 3; whichVertex++){
          int vertIndex = pObject.pFaces[j].vertIndex[whichVertex];
          gl.glNormal3f(pObject.pNormals[ vertIndex ].x, pObject.pNormals[ vertIndex ].y, pObject.pNormals[ vertIndex ].z);
          if(t3Dmodel.pMaterials.size() > 0 && pObject.materialID >= 0){
            float[] pColor = ((CLoadASE.tMaterialInfo)t3Dmodel.pMaterials.get(pObject.materialID)).fColor;
            if (withColor) gl.glColor3fv(pColor);
          }
          gl.glVertex3f(pObject.pVerts[ vertIndex ].x, pObject.pVerts[ vertIndex ].y, pObject.pVerts[ vertIndex ].z);
        }
      }
      gl.glEnd();
      gl.glEndList();  //fin de la liste
    }

  }

  //------------------------------------------------------------
  /** 
   * Supprime la liste d'affichage cre par la fonction prcdente
   */
  public static void delList(GLFunc gl, int typ){
    gl.glDeleteLists(index[typ],1); //on libre la mmoire
    index[typ] = -1;
  }

}

