//------------------------------------------------------------
/**
 * SimAntJFrame est l'IHM pour la simulation d'une fourmiliere
 *
 *
 * Copyright (C) 2004  Olivier Grunder
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; see the file COPYING.  If not, write to
 *   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *   Boston, MA 02111-1307, USA.
 *
 *   Contact: Olivier Grunder, 
 *            mailto:olivier.grunder@utbm.fr
 *            http://set.utbm.fr/membres/grunder/
 *
 *
 * @author      Olivier Grunder
 * @date        janvier 2004
 * @version     0.22
 */

   import javax.swing.*;
   import java.awt.*;
   import java.awt.event.*;

    public class SimAntJFrame extends JFrame implements KeyListener {
   
      private int nfl; // Nombre de fourmilires
      private int nfm; // Nombre de fourmis
      private int nn;  // Nombre de sources de nourriture
      private int no;  // Nombre d'obstacles
   
      public static final int MODE_TEXTE = 1 ;
      public static final int MODE_2D = 2 ;
      public static final int MODE_3D = 3 ;
      public static final int MODE_GL4JAVA = 4 ;
   
      Simulation simu ;
   
      private int modeAffichage ;
   
      private Thread threadAff2D ;
   
   /**
   * Vue 3D
   */
      private SimAntGLAnimCanvas simantGLAnimCanvas ;
   
   /**
   * Vue 2D
   */
      private SimAntJPanel simantJPanel ;
      private JScrollPane simantJScrollPane ;
   //------------------------------------------------------------
   /**
   * Constructeur
   * @param l      nombre de lignes de la matrice
   * @return          nombre de lignes de la matrice
   */
       public SimAntJFrame(String titre, int ma, int nfl0, int nfm0, int nn0, int no0){
         super(titre);
			
			nfl = nfl0;
			nfm = nfm0;
			nn = nn0;
			no = no0;
      
         modeAffichage = ma ;
      
      // ecouteur de la fermeture de la fenetre
         this.addWindowListener(
                new WindowAdapter(){
                   public void windowClosing(WindowEvent e){ System.exit(0); } } );
      
         if (modeAffichage==MODE_2D) {
            addKeyListener(this);
         }
      
      // Dimensionnement de la fenetre
         setSize (500,500) ;
      
      // Nouvelle simulation 
         simu = new Simulation(2000, 2000, nfl, nfm, nn, no) ;
         simu.start() ;
      
         switch (modeAffichage) {
            case MODE_TEXTE : 
               System.out.println ("Mode affichage texte") ;
               break ;
            case MODE_2D :
               initJFrame2D() ;
               break ;
            case MODE_3D :
            case MODE_GL4JAVA :
               initJFrame3D() ;
               break ;
         }
      
         show() ;
      }
   
   
   //------------------------------------------------------------
   /**
   * Init de la JFrame 2D 
   */
       public void initJFrame2D() {
         System.out.println ("Mode affichage 2D") ;
         simantJPanel = new SimAntJPanel (getSize().width, getSize().height,simu) ;
         simantJScrollPane = new JScrollPane (simantJPanel) ;
      
      // Taille visible du JScrollPane : on enleve 50 pour tenir compte de la taille
      // des barres de dfilement ( gauche et en bas)
         simantJScrollPane.setPreferredSize(new Dimension(getSize().width-50, getSize().height-50)) ;
      
         simantJPanel.setJScrollPane(simantJScrollPane) ;
         simantJPanel.calculerZoomVueDEnsemble() ;
      //    simantJPanel.setFacteurZoom(1.0) ;
         this.getContentPane().add(simantJScrollPane) ;
      
         threadAff2D = new Thread(simantJPanel) ;
         threadAff2D.start() ;
      }
   
   //------------------------------------------------------------
   /**
   * Init de la JFrame 3D 
   */
       public void initJFrame3D() {
         System.out.println ("Mode affichage 3D gl4java") ;
         simantGLAnimCanvas = new SimAntGLAnimCanvas(getSize().width, getSize().height,simu) ;
         this.getContentPane().add(simantGLAnimCanvas) ;
      }
   
   
   //------------------------------------------------------------
       public void keyTyped(KeyEvent e) {
      }
   
   //------------------------------------------------------------
   /** 
   * Ecouteur clavier
   * 
   * PROBLEME : Quand les scrolleurs sont actifs au dpart, les
   * vnements claviers ne sont plus traits. Il faut se passer
   * du JScrollPane ...
   */
       public void keyPressed(KeyEvent e) {
         switch(e.getKeyCode())
         {
            case KeyEvent.VK_ESCAPE: // Exit
               {
                  System.exit(0);
                  break;
               }
            case KeyEvent.VK_UP: // 
               {
                  break;
               }
            case KeyEvent.VK_DOWN:
               {
                  break;
               }
            case KeyEvent.VK_RIGHT:
               {
                  break;
               }
            case KeyEvent.VK_LEFT:
               {
                  break;
               }
            case KeyEvent.VK_GREATER : 
            case KeyEvent.VK_PLUS : 
            case 107 :
               {
                  break ;
               }
            case KeyEvent.VK_SUBTRACT : 
            case KeyEvent.VK_MINUS :
               {
                  break ;
               }
            case KeyEvent.VK_PAGE_DOWN: // variation d'altitude
               {
                  break;
               }
            case KeyEvent.VK_PAGE_UP:
               {
                  break;
               }
            case KeyEvent.VK_W: // Mode wireframe
               {
                  break;
               }
            case KeyEvent.VK_L: // Mode lighting
               {
                  break;
               }
            case KeyEvent.VK_T: // Mode texture
               {
                  break;
               }
            case KeyEvent.VK_F: // Mode fly
               {
                  break;
               }
            case KeyEvent.VK_R: // Mode Reinit
               {
                  simu.stop() ;
                  simu.reinit() ;
                  simu.start() ;
                  break;
               }
         
         }
      }
   
   //------------------------------------------------------------
       public void keyReleased(KeyEvent e) {
      }
   }
