//------------------------------------------------------------
/**
 * SimAntJPanel est la vue 2D pour la simulation d'une fourmiliere
 *
 *
 * Copyright (C) 2004  Olivier Grunder
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; see the file COPYING.  If not, write to
 *   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *   Boston, MA 02111-1307, USA.
 *
 *   Contact: Olivier Grunder, 
 *            mailto:olivier.grunder@utbm.fr
 *            http://set.utbm.fr/membres/grunder/
 *
 *
 * @author      Olivier Grunder
 * @date        janvier 2004
 * @version     0.22
 */

import java.util.* ;
import java.awt.*;
import java.awt.event.*;
import javax.swing.* ;

class SimAntJPanel extends JPanel implements MouseListener, MouseMotionListener, Runnable
{
  private static final double CONV_DEG_RAD = Math.PI/180.0 ;

  private JScrollPane jScrollPane ;
  private double facteurZoom = 1.0 ;

  private float largeur,profondeur ;

  private Simulation simu ;
  

  /** 
   * pour une gestion plus pousse de la souris
   */
  private int mousexold, mouseyold, mousex, mousey ;

  private double xreel, yreel ;

  //------------------------------------------------------------
  /** 
   * Constructeur 
   */
  public SimAntJPanel(int w, int h, Simulation sim)
  {
    //    super(w, h);
    simu = sim ;

    //on coute la souris 
    addMouseListener(this);
    addMouseMotionListener (this);
  }

  public void setJScrollPane (JScrollPane jsp) {jScrollPane = jsp ;}

  //------------------------------------------------------------
  /** 
   * Gestion de l'affichage 
   */
  synchronized public void paint(Graphics g) {
    // Efface la zone d'affichage
    g.setColor(new Color(0,0,0));
    g.fillRect(0, 0, getSize().width, getSize().height);
    //    g.setColor(new Color(Terrain.RED, Terrain.GREEN, Terrain.BLUE));
    g.setColor(Color.lightGray);
    g.fillRect(0, 0, (int)(simu.getLargeur()*facteurZoom), (int)(simu.getProfondeur()*facteurZoom) - 1);
	
    // Dessin des objets 2D
    int i = 0 ;
    // int imax = simu.getNbrObjets() ;
	 int imax = simu.getNbrObjetsVisibles() ;
    while (i<imax) {
      ObjetVisible obj = simu.getObjetVisible(i) ;
      obj.draw(g, facteurZoom) ;
      i++ ;
    }

    g.setColor(Color.white);
    g.fillRect((int)(xreel*facteurZoom), (int)(yreel*facteurZoom),
               (int)(2*(facteurZoom+1)), (int)(2*(facteurZoom+1))) ;
  }

  //--------------------------------
  /*
   * set facteurZoom
   */
  public void setFacteurZoom(double fz) { 
    facteurZoom = fz ;

    // Taille visible du jscrollpane
    jScrollPane.setPreferredSize(new Dimension(jScrollPane.getSize().width, jScrollPane.getSize().height)) ;

    // Taille visualisable du jpanel
    setPreferredSize(new Dimension((int)(simu.getLargeur()*facteurZoom), (int)(simu.getProfondeur()*facteurZoom)));

    revalidate();
  }
    
  public void calculerZoomNormal() {setFacteurZoom(1.0) ;}

  //--------------------------------
  /*
   * Calcul du facteur de zoom vue d'ensemble
   */
  public void calculerZoomVueDEnsemble() {
    double facteurZoomX=1, facteurZoomY=1 ;
    if (simu.getLargeur()>0) 
      facteurZoomX = (double)jScrollPane.getPreferredSize().width/simu.getLargeur() ;
    if (simu.getProfondeur()>0) 
      facteurZoomY = (double)jScrollPane.getPreferredSize().height/simu.getProfondeur() ;

    facteurZoom = facteurZoomX ;
    if (facteurZoomY<facteurZoom) facteurZoom = facteurZoomY ;
    
    // Taille visible du jscrollpane
    jScrollPane.setPreferredSize(new Dimension(jScrollPane.getSize().width, jScrollPane.getSize().height)) ;

    // Le viewport est mis a la taille du JScrollPane car le facteur de zoom
    // va compresser les coordonnees des points pour que le graphe entier soit
    // visible dans le JScrollPane.
    // Taille visualisable du jpanel
    setPreferredSize(new Dimension(jScrollPane.getSize().width, jScrollPane.getSize().height)); 

    revalidate();
  }

  //------------------------------------------------------------
  public void run() {
    boolean finished = false ;
    while (!finished) {
      repaint() ;
      try {Thread.sleep(40) ;} catch(Exception e) {}
    }
  }

  //------------------------------------------------------------
  public void mouseEntered(MouseEvent evt) {
    //    Component comp = evt.getComponent();
    //    if( comp.equals(this) ) {
    //    }
  }

  //------------------------------------------------------------
  /** 
   * Ecouteur souris
   */
  public void mouseDragged(MouseEvent evt){
    mousex=evt.getX();
    mousey=evt.getY();

    mousexold=mousex;
    mouseyold=mousey;
  }

  //------------------------------------------------------------
  public void mouseMoved(MouseEvent evt){
    // mmorisation des coord pour justesse des calculs  la reprise du drag 
    mousex=evt.getX();
    mousey=evt.getY();

    mousexold=mousex;
    mouseyold=mousey;
  }

  //------------------------------------------------------------
  public void mouseExited(MouseEvent evt){
  }

  //------------------------------------------------------------
  /**
   * PROBLEME du zoom quand on clique avec la souris : mal centr
   */
  public void mousePressed(MouseEvent evt){
    int x = evt.getX() ;
    int y = evt.getY() ;

    JViewport viewport = jScrollPane.getViewport() ;
    Point ptreelaveczoom = viewport.toViewCoordinates(new Point (x,y)) ;
    
    xreel = ptreelaveczoom.getX()/facteurZoom ;
    yreel = ptreelaveczoom.getY()/facteurZoom ;

    if ((evt.getModifiers() & MouseEvent.BUTTON1_MASK)!=0){
      setFacteurZoom(2.0*facteurZoom) ;
    }
    else {
      setFacteurZoom(facteurZoom/2.0) ;
    }

    // facteurZoom a chang...
    Point ptviewposition = new Point ((int)(xreel*facteurZoom-(jScrollPane.getPreferredSize().width/4)/facteurZoom),
                                      (int)(yreel*facteurZoom-(jScrollPane.getPreferredSize().height/2)/facteurZoom)) ;

    viewport.setViewPosition(ptviewposition) ;
  }

  //------------------------------------------------------------
  public void mouseReleased(MouseEvent evt){
  }

  //------------------------------------------------------------
  public void mouseClicked(MouseEvent evt)
  {
    //    Component comp = evt.getComponent();
    //    if( comp.equals(this) ) {
    //    }
  }

}

