//------------------------------------------------------------
/**
 * Terrain dessine le terrain en 3D
 *
 *
 * Copyright (C) 2004  Olivier Grunder
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; see the file COPYING.  If not, write to
 *   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *   Boston, MA 02111-1307, USA.
 *
 *   Contact: Olivier Grunder, 
 *            mailto:olivier.grunder@utbm.fr
 *            http://set.utbm.fr/membres/grunder/
 *
 *
 * @author      Olivier Grunder
 * @date        janvier 2004
 * @version     0.22
 */

import gl4java.*;
import gl4java.utils.textures.*;

public class TerrainGL extends Terrain {
  private int[] texture ;
  private SimAntGLAnimCanvas simAntGLAnimCanvas ;

  //------------------------------------------------------------
  /** 
   * Constructeur par dfaut
   */
  public TerrainGL(SimAntGLAnimCanvas sim, int larg,int prof, GLFunc gl, GLUFunc glu){
    super(larg, prof) ;
    simAntGLAnimCanvas = sim ;

    // Creation d'un tableau d'index pour les textures
    texture = new int[1];

    // Creation de la texture du terrain
    if(!LoadGLTextures(gl, glu)){   
      System.out.println("Problme lors du chargement de la texture du terrain");
      System.exit(0);
    }
  }

  //------------------------------------------------------------
  /** 
   * Chargement de la texture du terrain
   */
  public boolean LoadGLTextures(GLFunc gl, GLUFunc glu){  
    // chargement du fichier texture
    PngTextureLoader texLoader = new PngTextureLoader(gl, glu);
    texLoader.readTexture("data/texture-terrain.png");

    if(texLoader.isOk()){
      //Creation de l'objet texture OpenGL
      gl.glGenTextures(1, texture); 
      //gnration d'un index
      gl.glBindTexture(gl.GL_TEXTURE_2D, texture[0]);
      gl.glTexParameteri(gl.GL_TEXTURE_2D, gl.GL_TEXTURE_MAG_FILTER, gl.GL_LINEAR);
      gl.glTexParameteri(gl.GL_TEXTURE_2D, gl.GL_TEXTURE_MIN_FILTER, gl.GL_LINEAR);
      gl.glTexImage2D(gl.GL_TEXTURE_2D,
                      0,                        //niveau de mipmap 0
                      3,                        //mode RGB : 3 composantes
                      texLoader.getImageWidth(),
                      texLoader.getImageHeight(),
                      0,                        //pas de bordure
                      gl.GL_RGB,
                      gl.GL_UNSIGNED_BYTE,
                      texLoader.getTexture());
      return true;
    }
    return false;
  }

  //------------------------------------------------------------
  /** 
   * Dessin du terrain
   */
  public void draw(GLFunc gl) {
    boolean istext = simAntGLAnimCanvas.isModeTexture() ;
    boolean islight = simAntGLAnimCanvas.isModeTexture() ;
    boolean iswire = simAntGLAnimCanvas.isModeWireFrame() ;

    gl.glDisable(gl.GL_LIGHTING);

    // Activation de la texture si besoin
    if(istext) {
      // Selection de la texture si elle existe
      gl.glBindTexture(gl.GL_TEXTURE_2D, texture[0]);
      gl.glEnable(gl.GL_TEXTURE_2D);
      gl.glColor3f((float)1.0,(float)1.0,(float)1.0);
    }
    else {
      gl.glDisable(gl.GL_TEXTURE_2D);
      gl.glColor3f(RED, GREEN, BLUE);
    }

    if (iswire) {
      for (float x=0;x<largeur;x+=maille) {
        for (float z=0;z<profondeur;z+=maille) {
          gl.glBegin(gl.GL_QUADS);
          gl.glVertex3f(x,0,z);
          gl.glVertex3f(x,0,z+maille);
          gl.glVertex3f(x+maille,0,z+maille);
          gl.glVertex3f(x+maille,0,z);
          gl.glEnd();
        }
      }
    }
    else {
      gl.glBegin(gl.GL_QUADS);
      gl.glTexCoord2f(4.0f, 4.0f);
      gl.glVertex3f(0,0,0);
      gl.glTexCoord2f(4.0f,0.0f);
      gl.glVertex3f(0,0,profondeur);
      gl.glTexCoord2f(0.0f,0.0f);
      gl.glVertex3f(largeur,0,profondeur);
      gl.glTexCoord2f(0.0f,4.0f);
      gl.glVertex3f(largeur,0,0);
      gl.glEnd();
    }
  }
}



